@@ -200,17 +200,19 @@ class Result:
         ''' Implements just enough of a stem RouterStatusEntryV3 for this
         Result class to be happy '''
         def __init__(self, fingerprint, nickname, address, master_key_ed25519,
-                     average_bandwidth=None):
+                     average_bandwidth=None, observed_bandwidth=None):
             self.fingerprint = fingerprint
             self.nickname = nickname
             self.address = address
             self.master_key_ed25519 = master_key_ed25519
             self.average_bandwidth = average_bandwidth
+            self.observed_bandwidth = observed_bandwidth
 
     def __init__(self, relay, circ, dest_url, scanner_nick, t=None):
         self._relay = Result.Relay(relay.fingerprint, relay.nickname,
                                    relay.address, relay.master_key_ed25519,
-                                   relay.average_bandwidth)
+                                   relay.average_bandwidth,
+                                   relay.observed_bandwidth)
         self._circ = circ
         self._dest_url = dest_url
         self._scanner = scanner_nick
@@ -224,6 +226,10 @@ class Result:
     def relay_average_bandwidth(self):
         return self._relay.average_bandwidth
 
+    @property
+    def relay_observed_bandwidth(self):
+        return self._relay.observed_bandwidth
+
     @property
     def fingerprint(self):
         return self._relay.fingerprint
@@ -470,10 +476,11 @@ class ResultSuccess(Result):
     def from_dict(d):
         assert isinstance(d, dict)
         return ResultSuccess(
-            d['rtts'], d['downloads'],
+            d['rtts'] or [], d['downloads'],
             Result.Relay(
                 d['fingerprint'], d['nickname'], d['address'],
-                d['master_key_ed25519'], d['relay_average_bandwidth']),
+                d['master_key_ed25519'], d['relay_average_bandwidth'],
+                d['relay_observed_bandwidth']),
             d['circ'], d['dest_url'], d['scanner'],
             t=d['time'])
 
@@ -483,6 +490,7 @@ class ResultSuccess(Result):
             'rtts': self.rtts,
             'downloads': self.downloads,
             'relay_average_bandwidth': self.relay_average_bandwidth,
+            'relay_observed_bandwidth': self.relay_observed_bandwidth,
         })
         return d
 
@@ -491,11 +499,11 @@ class ResultDump:
     ''' Runs the enter() method in a new thread and collects new Results on its
     queue. Writes them to daily result files in the data directory '''
     def __init__(self, args, conf, end_event):
-        assert os.path.isdir(conf['paths']['datadir'])
+        assert os.path.isdir(conf.getpath('paths', 'datadir'))
         assert isinstance(end_event, Event)
         self.conf = conf
         self.fresh_days = conf.getint('general', 'data_period')
-        self.datadir = conf['paths']['datadir']
+        self.datadir = conf.getpath('paths', 'datadir')
         self.end_event = end_event
         self.data = {}
         self.data_lock = RLock()
