@@ -1,3 +1,5 @@
+"""Util functions to manage sbws configuration files."""
+
 from configparser import (ConfigParser, ExtendedInterpolation)
 from configparser import InterpolationMissingOptionError
 import os
@@ -6,7 +8,9 @@ import logging.config
 from urllib.parse import urlparse
 from string import Template
 from tempfile import NamedTemporaryFile
-from sbws.globals import PKG_DIR
+from sbws.globals import (DEFAULT_CONFIG_PATH, DEFAULT_LOG_CONFIG_PATH,
+                          USER_CONFIG_PATH, SUPERVISED_RUN_DPATH,
+                          SUPERVISED_USER_CONFIG_PATH, fail_hard)
 
 _ALPHANUM = 'abcdefghijklmnopqrstuvwxyz'
 _ALPHANUM += _ALPHANUM.upper()
@@ -21,8 +25,16 @@ _LOG_LEVELS = ['debug', 'info', 'warning', 'error', 'critical']
 log = logging.getLogger(__name__)
 
 
-def _read_config_file(conf, fname):
-    assert os.path.isfile(fname)
+def _expand_path(path):
+    """Expand path string containing shell variables and ~ constructions
+    into their values. Environment variables have to have their $ escaped by
+    another $. For example: $$XDG_RUNTIME_DIR/foo.bar
+    """
+    return os.path.expanduser(os.path.expandvars(path))
+
+
+def _extend_config(conf, fname):
+    """Extend ConfigParser from file configuration."""
     log.debug('Reading config file %s', fname)
     with open(fname, 'rt') as fd:
         conf.read_file(fd, source=fname)
@@ -30,51 +42,57 @@ def _read_config_file(conf, fname):
 
 
 def _get_default_config():
-    conf = ConfigParser(interpolation=ExtendedInterpolation())
-    fname = os.path.join(PKG_DIR, 'config.default.ini')
-    assert os.path.isfile(fname)
-    conf = _read_config_file(conf, fname)
-    return conf
+    """Return ConfigParser with default configuration."""
+    conf = ConfigParser(interpolation=ExtendedInterpolation(),
+                        converters={'path': _expand_path})
+    return _extend_config(conf, DEFAULT_CONFIG_PATH)
+
+
+def _obtain_user_conf_path():
+    if os.environ.get("SUPERVISED") == "1":
+        return SUPERVISED_USER_CONFIG_PATH
+    return USER_CONFIG_PATH
 
 
 def _get_user_config(args, conf=None):
+    """Get user configuration.
+    Search for user configuration in the default path or the path passed as
+    argument and extend the configuration if they are found.
+    """
     if not conf:
-        conf = ConfigParser(interpolation=ExtendedInterpolation())
+        conf = ConfigParser(interpolation=ExtendedInterpolation(),
+                            converters={'path': _expand_path})
     else:
         assert isinstance(conf, ConfigParser)
-    fname = os.path.join(args.directory, 'config.ini')
-    if not os.path.isfile(fname):
-        return conf
-    conf = _read_config_file(conf, fname)
+    if args.config:
+        if not os.path.isfile(args.config):
+            fail_hard('Configuration file %s not found.', args.config)
+        return _extend_config(conf, args.config)
+    user_config_path = _obtain_user_conf_path()
+    if os.path.isfile(user_config_path):
+        return _extend_config(conf, user_config_path)
+    log.debug('No user config found.')
     return conf
 
 
-def _get_default_logging_config(args, conf=None):
+def _get_default_logging_config(conf=None):
+    """Get default logging configuration."""
     if not conf:
-        conf = ConfigParser(interpolation=ExtendedInterpolation())
+        conf = ConfigParser(interpolation=ExtendedInterpolation(),
+                            converters={'path': _expand_path})
     else:
         assert isinstance(conf, ConfigParser)
-    fname = os.path.join(PKG_DIR, 'config.log.default.ini')
-    assert os.path.isfile(fname)
-    conf = _read_config_file(conf, fname)
-    return conf
+    return _extend_config(conf, DEFAULT_LOG_CONFIG_PATH)
 
 
 def get_config(args):
+    """Get ConfigParser interpolating all configuration files."""
     conf = _get_default_config()
-    conf = _get_default_logging_config(args, conf=conf)
+    conf = _get_default_logging_config(conf=conf)
     conf = _get_user_config(args, conf=conf)
     return conf
 
 
-def get_user_example_config():
-    conf = ConfigParser(interpolation=ExtendedInterpolation())
-    fname = os.path.join(PKG_DIR, 'config.example.ini')
-    assert os.path.isfile(fname)
-    conf = _read_config_file(conf, fname)
-    return conf
-
-
 def _can_log_to_file(conf):
     '''
     Checks all the known reasons for why we might not be able to log to a file,
@@ -90,7 +108,7 @@ def _can_log_to_file(conf):
     # If there is an issue getting this option, tell the caller that we can't
     # log to file.
     try:
-        conf['paths']['log_dname']
+        conf.getpath('paths', 'log_dname')
     except InterpolationMissingOptionError as e:
         return False, e
     return True, ''
@@ -122,15 +140,18 @@ def configure_logging(args, conf):
         # The first argument is the file name to which it should log. Set it to
         # the sbws command (like 'scanner' or 'generate') if possible, or to
         # 'sbws' failing that.
-        dname = conf['paths']['log_dname']
+        dname = conf.getpath('paths', 'log_dname')
         os.makedirs(dname, exist_ok=True)
         fname = os.path.join(dname, '{}.log'.format(args.command or 'sbws'))
-        # The second argument is the number of backups to keep, and the third
-        # is the maximum file size (in bytes) each log file should be.
+        # The second argument is the file mode, and it should be left alone
+        mode = 'a'
+        # The third is the maximum file size (in bytes) each log file should be
         max_bytes = conf.getint('logging', 'to_file_max_bytes')
+        # And the forth is the number of backups to keep
         num_backups = conf.getint('logging', 'to_file_num_backups')
         # Now store those things as a string in the config. So dumb.
-        conf['handler_to_file']['args'] = str((fname, num_backups, max_bytes))
+        conf['handler_to_file']['args'] = \
+            str((fname, mode, max_bytes, num_backups))
     # Set some stuff that needs config parser's interpolation
     conf['formatter_to_file']['format'] = conf['logging']['to_file_format']
     conf['formatter_to_stdout']['format'] = conf['logging']['to_stdout_format']
@@ -169,8 +190,10 @@ def _validate_cleanup(conf):
     sec = 'cleanup'
     err_tmpl = Template('$sec/$key ($val): $e')
     ints = {
-        'stale_days': {'minimum': 1, 'maximum': None},
-        'rotten_days': {'minimum': 1, 'maximum': None},
+        'data_files_compress_after_days': {'minimum': 1, 'maximum': None},
+        'data_files_delete_after_days': {'minimum': 1, 'maximum': None},
+        'v3bw_files_compress_after_days': {'minimum': 1, 'maximum': None},
+        'v3bw_files_delete_after_days': {'minimum': 1, 'maximum': None},
     }
     all_valid_keys = list(ints.keys())
     errors.extend(_validate_section_keys(conf, sec, all_valid_keys, err_tmpl))
@@ -202,13 +225,30 @@ def _validate_general(conf):
     return errors
 
 
+def _obtain_sbws_home(conf):
+    sbws_home = conf.getpath('paths', 'sbws_home')
+    # No need for .sbws when this is the default home
+    if sbws_home == "/var/lib/sbws/.sbws":
+        conf['paths']['sbws_home'] = os.path.dirname(sbws_home)
+
+
+def _obtain_run_dpath(conf):
+    """Set runtime directory when sbws is run by a system service."""
+    xdg = os.environ.get('XDG_RUNTIME_DIR')
+    if os.environ.get('SUPERVISED') == "1":
+        conf['tor']['run_dpath'] = SUPERVISED_RUN_DPATH
+    elif xdg is not None:
+        conf['tor']['run_dpath'] = os.path.join(xdg, 'sbws', 'tor')
+
+
 def _validate_paths(conf):
+    _obtain_sbws_home(conf)
     errors = []
     sec = 'paths'
     err_tmpl = Template('$sec/$key ($val): $e')
     unvalidated_keys = [
-        'datadir', 'sbws_home', 'v3bw_fname', 'v3bw_dname',
-        'started_filepath', 'log_dname']
+        'datadir', 'sbws_home', 'v3bw_fname', 'v3bw_dname', 'state_fname',
+        'log_dname']
     all_valid_keys = unvalidated_keys
     allow_missing = ['sbws_home']
     errors.extend(_validate_section_keys(conf, sec, all_valid_keys, err_tmpl,
@@ -221,7 +261,7 @@ def _validate_scanner(conf):
     sec = 'scanner'
     err_tmpl = Template('$sec/$key ($val): $e')
     ints = {
-        'num_rtts': {'minimum': 1, 'maximum': 100},
+        'num_rtts': {'minimum': 0, 'maximum': 100},
         'num_downloads': {'minimum': 1, 'maximum': 100},
         'initial_read_request': {'minimum': 1, 'maximum': None},
         'measurement_threads': {'minimum': 1, 'maximum': None},
@@ -247,11 +287,12 @@ def _validate_scanner(conf):
 
 
 def _validate_tor(conf):
+    _obtain_run_dpath(conf)
     errors = []
     sec = 'tor'
     err_tmpl = Template('$sec/$key ($val): $e')
     unvalidated_keys = [
-        'datadir', 'control_socket', 'log', 'extra_lines']
+        'datadir', 'run_dpath', 'control_socket', 'pid', 'log', 'extra_lines']
     all_valid_keys = unvalidated_keys
     errors.extend(_validate_section_keys(conf, sec, all_valid_keys, err_tmpl))
     return errors
